package com 
{
	import org.flixel.*;
	
	/**
	 * ...
	 * @author Lulzy & Villermen
	 */
	public class MapSegment extends FlxGroup 
	{
		/**
		 * The <code>FlxTilemap</code> containing the collidable blocks in this segment
		 */
		public var map:FlxTilemap;
		
		/**
		 * The <code>FlxTilemap</code> containing decoration blocks
		 */
		public var decoMap:FlxTilemap;
		
		/**
		 * The <code>FlxTilemap</code> containing the objects in this segment
		 */
		private var _objectMap:FlxTilemap;	
		
		/**
		 * The <code>FlxGroup</code> containing the pipes in this segment
		 */
		public var pipeGroup:FlxGroup;
		
		/**
		 * The <code>FlxGroup</code> containing the oneway blocks in this segment
		 */
		public var onewayGroup:FlxGroup;
		
		/**
		 * The <code>FlxGroup</code> containing the acid in this segment
		 */
		public var acidGroup:FlxGroup;
		
		/**
		 * The <code>FlxGroup</code> containing the ladders in this segment
		 */
		public var ladderGroup:FlxGroup;
		
		/**
		 * The <code>FlxGroup</code> containing the lead items in this segment
		 */
		public var leadGroup:FlxGroup;
		
		/**
		 * The <code>FlxGroup</code> containing springs in this segment
		 */
		public var springGroup:FlxGroup;
		
		/**
		 * <code>FlxGroup</code> containing all other object groups for quick access
		 */
		public var objectsGroup:FlxGroup;
		
		/**
		 * Background image
		 */
		public var bgImg:FlxSprite;
		 
		// Tilemap images
		private var _mapTiles:Class;
		private var _objectTiles:Class;
		private var _decoTiles:Class;
		
		private var _mapData:String;
		private var _objectData:String;
		private var _decoData:String;
		
		public var _offSetX:Number;
		
		public function MapSegment(mapData:String, objectData:String, decoData:String, offSetX:Number = 0) 
		{			
			// the main map
			map = new FlxTilemap();
			
			// map containing objects
			_objectMap = new FlxTilemap();
			
			// store offset locally
			_offSetX = offSetX;
			
			// object groups
			pipeGroup = new FlxGroup();
			onewayGroup = new FlxGroup();
			acidGroup = new FlxGroup();
			ladderGroup = new FlxGroup();
			leadGroup = new FlxGroup();
			springGroup = new FlxGroup();
			
			objectsGroup = new FlxGroup();
			objectsGroup.add(pipeGroup);
			objectsGroup.add(onewayGroup);
			objectsGroup.add(acidGroup);
			objectsGroup.add(ladderGroup);
			objectsGroup.add(leadGroup);
			objectsGroup.add(springGroup);
			
			// add everything
			add(map);
			add(pipeGroup);
			add(onewayGroup);
			add(ladderGroup);
			add(acidGroup);
			add(leadGroup);
			add(springGroup);
			
			// map with decoration tiles
			decoMap = new FlxTilemap;
			add(decoMap);
			
			// tile images
			_mapTiles = Assets.mapTiles;
			_objectTiles = Assets.objectTiles;
			_decoTiles = Assets.decoTiles;
			
			// CSV data in strings
			_mapData = mapData;
			_objectData = objectData;
			_decoData = decoData;
			
			// load everything up
			loadMap();
			loadObjects();
			loadDecos();
		}
		
		public function loadMapData(mapData:String):void
		{
			_mapData = mapData;
			
			loadMap();
		}
		
		public function loadObjectData(objectData:String, loadLead:Boolean = true):void
		{
			_objectData = objectData;
			
			loadObjects(loadLead);
		}
		
		public function loadDecoData(decoData:String):void
		{
			_decoData = decoData;
			
			loadDecos();
		}
		
		private function loadMap():void
		{
			map.loadMap(_mapData, _mapTiles, 20, 20);
			map.x = _offSetX;
		}
		
		public function loadObjects(loadLead:Boolean = true):void
		{
			_objectMap.loadMap(_objectData, _objectTiles, 20, 20);
			objectsGroup.callAll("kill");
			
			// loop through all tile types
			for (var i:uint = 1; i <= 19; i++)
			{
				// array with found objects of type
				var objects:Array = _objectMap.getTileCoords(i, false);
				
				// if tile type found
				if (objects != null)
				{
					// loop through all tiles of type
					for (var j:uint = 0; j < objects.length; j++ )
					{
						var object:FlxSprite;
						// oneways
						if (i >= 1 && i <= 4)
						{
							object = new FlxSprite(objects[j].x, objects[j].y);
							object.x += _offSetX;
							object.loadGraphic(_objectTiles, false, false, 20, 20);
							object.immovable = true;
							object.frame = i;
							object.allowCollisions = FlxObject.UP;
							onewayGroup.add(object);
						}
						// pipes
						if (i >= 5 && i <= 9)
						{
							object = new FlxSprite(objects[j].x, objects[j].y);
							object.x += _offSetX;
							object.loadGraphic(_objectTiles, false, false, 20, 20);
							object.immovable = true;
							object.frame = i;
							object.allowCollisions = FlxObject.DOWN;
							pipeGroup.add(object);
						}
						// ladders
						if (i == 10)
						{
							object = new FlxSprite(objects[j].x, objects[j].y);
							object.x += _offSetX;
							object.loadGraphic(_objectTiles, false, false, 20, 20);
							object.immovable = true;
							object.frame = i;
							ladderGroup.add(object);
						}
						// acid
						if (i == 11 || i == 12 || i == 15)
						{
							object = new FlxSprite(objects[j].x, objects[j].y);
							object.x += _offSetX;
							object.loadGraphic(_objectTiles, false, false, 20, 20);
							object.immovable = true;
							object.frame = i;
							acidGroup.add(object);
						}
						// lead
						if (i == 13 && loadLead)
						{
							var lead:LeadUpgrade = new LeadUpgrade(objects[j].x, objects[j].y);
							lead.x += _offSetX;
							leadGroup.add(lead);
						}
						// spring
						if (i == 14)
						{
							var spring:Spring = new Spring(objects[j].x, objects[j].y);
							spring.x += _offSetX;
							spring.y -= 20;
							springGroup.add(spring);
						}
						
					}
				}
			}
			
		}
		
		public function loadDecos():void
		{
			decoMap.loadMap(_decoData, _decoTiles, 20, 20);
			decoMap.x = _offSetX;
		}
		
	}

}