package com 
{
	import org.flixel.*;
	
	/**
	 * ...
	 * @author Lulzy & Villermen
	 */
	public class PlayState extends FlxState 
	{		
		/**
		 * Player instance
		 */
		public static var player:Player;
		 
		// Call assets constructor to build some data, MUST HAPPEN BEFORE THE MAPLOADER
		private var assets:Assets = new Assets();
		
		// Tile map loader (HUGE HACK)	
		public var mapLoader:MapLoader = new MapLoader();
		
		/**
		 * Textfield displaying time left for the player
		 */
		public var timeText:FlxText;
		
		/**
		 * Shows the score of the player
		 */
		public var scoreText:FlxText;
		
		/**
		 * Text telling the player when the game is over
		 */
		public var gameOverText:FlxText;
		
		/**
		 * Flag indicating if the game has ended
		 */
		public var gameOver:Boolean = false;
		
		/**
		 * Green overlay when the player is running out of time
		 */
		public var radiationOverlay:FlxSprite;
		
		/**
		 * The sound object containing the music intro
		 */
		public var musicIntro:Sound;
		
		/**
		 * The sound object containing the looped song
		 */
		public var musicLoop:Sound;
		
		/**
		 * Text showing when you pick up lead
		 */
		public var leadText:FlxText;
		
		/**
		 * The background images
		 */
		public var bgImg1:FlxSprite;
		public var bgImg2:FlxSprite;
		public var bgImg3:FlxSprite;
		
		public function PlayState() 
		{
			super();
			
			initBackgrounds();
			add(bgImg1);
			add(bgImg2);
			add(bgImg3);
			
			add(mapLoader);
			
			// add the player
			player = new Player();
			add(player);
			
			// music
			musicLoop = new Sound();
			musicLoop.loadEmbedded(Assets.loopMusic, true, false);
			musicIntro = new Sound();
			musicIntro.loadEmbedded(Assets.introMusic, false);
			musicIntro.play();
			add(musicLoop);
			add(musicIntro);
			
			radiationOverlay = new FlxSprite(0, 0);
			radiationOverlay.makeGraphic(800, 480, FlxG.GREEN);
			radiationOverlay.scrollFactor = new FlxPoint(0, 0);
			radiationOverlay.alpha = 0;
			add(radiationOverlay);
			
			timeText = new FlxText(5, 64, 50, "10");
			timeText.solid = false;
			timeText.scrollFactor = new FlxPoint(0, 0);
			timeText.size = 16;
			add(timeText);
			
			scoreText = new FlxText(700, 64, 100, "0");
			scoreText.solid = false;
			scoreText.scrollFactor = new FlxPoint(0, 0);
			scoreText.size = 16;
			add(scoreText);
			
			leadText = new FlxText(player.x - 116, player.y - 20, 200, "Lead get! +10s time");
			leadText.alignment = "center";
			leadText.alpha = 0;
			add(leadText);
			
			FlxG.resetCameras();
			FlxG.bgColor = FlxG.BLACK;
		}
		
		override public function update():void
		{
			if (FlxG.keys.pressed("P"))
			{
				FlxG.paused = true;
			}
			
			if (player.alive)
			{
				// PLAY MUSIC
				if (!musicIntro.active)
				{
					musicLoop.play();
				}
				
				// check roof
				if (player.y <= 39)
				{
					player.velocity.y = 0;
					player.y = 40;
				}
				
				// Collide with map and check if on floor
				if (FlxG.collide(player, mapLoader.currentSegment.map) && player.velocity.y == 0)
					player.onFloor = true;
				else
					player.onFloor = false;
				
				// Check oneway platforms
				player.allowCollisions = FlxObject.FLOOR;
				if (player.velocity.y > 0)
				{
					if (FlxG.collide(player, mapLoader.currentSegment.onewayGroup))
					{
						player.onFloor = true;
					}
				}
				
				// Check pipes
				player.allowCollisions = FlxObject.CEILING;
				if (FlxG.overlap(player, mapLoader.currentSegment.pipeGroup) && player.velocity.y <= 0)
				{
					player.hanging = true;
				}
				else
				{
					player.hanging = false;
				}
				
				player.allowCollisions = FlxObject.ANY;
				// Check ladders
				if (FlxG.overlap(player, mapLoader.currentSegment.ladderGroup))
				{
					player.onLadder = true;
				}
				else
				{
					player.onLadder = false;
				}
				
				// Check acid
				if (FlxG.collide(player, mapLoader.currentSegment.acidGroup) && player.alive)
				{
					player.alive = false;
					player.play("ACID_DEATH_RIGHT");
				}
				
				// check lead
				if (FlxG.overlap(player, mapLoader.currentSegment.leadGroup))
				{
					leadText.y = player.y - 20;
					leadText.alpha = 1;
					player.timeLeft += 10;
				}
				
				// check springs
				if (FlxG.overlap(player, mapLoader.currentSegment.springGroup))
				{
					player.velocity.y = -600;
				}
			}
			
			if (player.timeLeft < 5)
				radiationOverlay.alpha = 0.25 - 0.05 * player.timeLeft
			else
				radiationOverlay.alpha = 0;
			
			FlxG.camera.setBounds(-800, 0, 99999, 480);
			FlxG.camera.follow(player);
			
			timeText.text = Math.ceil(player.timeLeft).toString();
			scoreText.text = Math.round(player.score).toString();
			
			if ((player.timeLeft <= 0 || !player.alive) && !gameOver)
			{
				musicIntro.fadeOut(1);
				musicLoop.fadeOut(1);
				gameOver = true;
				player.alive = false;
				
				if (player.facing == FlxObject.RIGHT)
					player.play("ACID_DEATH_RIGHT");
				else
					player.play("ACID_DEATH_LEFT");
					
				gameOverText = new FlxText(150, 240, 500, "You died.\nYour lived for: " + Math.round(player.score).toString() + " seconds\n Press space to restart");
				gameOverText.alignment = "center";
				gameOverText.scrollFactor = new FlxPoint();
				gameOverText.size = 24;
				add(gameOverText);
				
				player.moves = false;
			}
			if (FlxG.keys.pressed("SPACE") && gameOver)
			{
				FlxG.resetState();
				musicIntro.kill();
				musicLoop.kill();
			}
			
			if (leadText.alpha > 0)
			{
				leadText.x = player.x - 100;
				leadText.y -= 0.5;
				leadText.alpha -= 0.01;
			}
			
			super.update();
			
		}
		
		public function initBackgrounds():void
		{
			bgImg1 = new FlxSprite();
			bgImg2 = new FlxSprite();
			bgImg3 = new FlxSprite();
			
			bgImg1.loadGraphic(Assets.bgImg, false, false, 800, 480);
			bgImg2.loadGraphic(Assets.bgImg, false, false, 800, 480);
			bgImg3.loadGraphic(Assets.bgImg, false, false, 800, 480);
			
			bgImg1.solid = false;
			bgImg2.solid = false;
			bgImg3.solid = false;
			
			bgImg1.x = -800;
			bgImg3.x = 800;
		}
	}

}