package com 
{
	import org.flixel.*;
	
	/**
	 * ...
	 * @author Lulzy & Villermen
	 */
	public class Player extends FlxSprite 
	{
		/**
		 * Gravity for the player
		 */
		public const GRAVITY:uint = 500;
		
		/**
		 * Movement the player makes this frame
		 */
		private var _movement:FlxPoint = new FlxPoint(0, 0);
		
		/**
		 * Boolean flag indicating whether the player is touching the floor
		 */
		public var onFloor:Boolean = false;
		
		/**
		 * Flag indicating if player is hanging from a pipe
		 */
		public var hanging:Boolean = false;
		
		/**
		 * Flag indicating if player is climbing a ladder
		 */
		public var onLadder:Boolean = false;
		
		/**
		 * Radiation timer
		 */
		public var timeLeft:Number = 10;
		
		/**
		 * Player score
		 */
		public var score:Number = 0;
		
		public function Player() 
		{
			loadGraphic(Assets.playerAnim, true, false, 32, 60);
			
			x = 100;
			y = 360;
			
			acceleration.y = GRAVITY;
			maxVelocity.y = 1000;
			
			addAnimation("WALK_RIGHT", [0, 1, 2, 3, 4, 5, 6, 7], 16);
			addAnimation("LOOK_RIGHT", [0]);
			addAnimation("WALK_LEFT", [8, 9, 10, 11, 12, 13, 14, 15], 16);
			addAnimation("LOOK_LEFT", [8]);
			addAnimation("JUMP_RIGHT", [24]);
			addAnimation("JUMP_LEFT", [25]);
			addAnimation("CLIMB_LEFT", [20, 21, 22, 23], 8);
			addAnimation("CLIMB_RIGHT", [16, 17, 18, 19], 8);
			addAnimation("FALL_LEFT", [27]);
			addAnimation("FALL_RIGHT", [26]);
			addAnimation("CLIMB_IDLE_LEFT", [20]);
			addAnimation("CLIMB_IDLE_RIGHT", [16]);
			addAnimation("ACID_DEATH_RIGHT", [28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45], 8, false);
			addAnimation("ACID_DEATH_LEFT", [46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63], 8, false);
			addAnimation("LADDER_UP", [64, 65, 66, 67, 68], 10);
			addAnimation("LADDER_DOWN", [68, 67, 66, 65, 64], 10);
			addAnimation("LADDER_IDLE", [64]);
		}
		
		override public function update():void
		{
			if (alive)
			{
				_movement = new FlxPoint(0, 0);
				
				if (FlxG.keys.pressed("D") || FlxG.keys.pressed("RIGHT"))
				{
					_movement.x += 2;
				}
				if (FlxG.keys.pressed("A") || FlxG.keys.pressed("LEFT"))
				{
					_movement.x -= 2;
				}
				if ((FlxG.keys.pressed("W") || FlxG.keys.pressed("UP")) && onLadder)
				{
					_movement.y -= 1;
				}
				else if ((FlxG.keys.justPressed("W") || FlxG.keys.justPressed("UP") || FlxG.keys.justPressed("SPACE")) && onFloor)
				{
					velocity.y = -300;
					//FlxG.play(Assets.jumpSound, 0.25);
					onFloor = false;
					hanging = false;
				}
				if (FlxG.keys.pressed("S") || FlxG.keys.pressed("DOWN"))
				{
					if (onLadder)
					{
						_movement.y += 1;
						
					}
					if (hanging)
					{
						hanging = false;
					}
				}
				
				if (hanging || onLadder)
				{
					velocity.y = 0;
					acceleration.y = 0;
				}
				else
				{
					acceleration.y = GRAVITY;
				}
				
				this.x += _movement.x;
				this.y += _movement.y;
				
				if (!Main.tutorial)
				{
					timeLeft -= 1 / 60;
					score += 1 / 60;
				}
			}
			
			super.update();
		}
		
		override protected function updateAnimation():void
		{
			if (alive)
			{
				if (_movement.x > 0)
					facing = RIGHT;
				if (_movement.x < 0)
					facing = LEFT;
				
				if (!hanging && facing == RIGHT)
				{
					if (onFloor)
					{
						play("WALK_RIGHT");
					}
					else if (velocity.y > 0)
					{
						play("FALL_RIGHT");
					}
					else if (velocity.y < 0)
					{
						play("JUMP_RIGHT");
					}
				}
				if (!hanging && facing == LEFT)
				{
					if (onFloor)
					{
						play("WALK_LEFT");
					}
					else if (velocity.y > 0)
					{
						play("FALL_LEFT");
					}
					else if (velocity.y < 0)
					{
						play("JUMP_LEFT");
					}
				}
				if (_movement.x == 0 && onFloor)
				{
					if (facing == RIGHT)
						play("LOOK_RIGHT");
					if (facing == LEFT)
						play("LOOK_LEFT");
				}
				if (facing == RIGHT && hanging)
				{
					play("CLIMB_RIGHT");
					if (_movement.x == 0)
						play("CLIMB_IDLE_RIGHT");
				}
				if (facing == LEFT && hanging)
				{
					play("CLIMB_LEFT");
					if (_movement.x == 0)
						play("CLIMB_IDLE_LEFT");
				}
				if (onLadder)
				{
					if (_movement.y == 0)
						play("LADDER_IDLE");
					if (_movement.x > 0)
						play("LADDER_DOWN");
					if (_movement.x < 0)
						play("LADDER_UP");
					if (_movement.y > 0)
						play("LADDER_DOWN");
					if (_movement.y < 0)
						play("LADDER_UP");
				}
			}
			
			super.updateAnimation();
		}
	
	
	}

}